#include "kernel/include/types.h"
#include "kernel/include/stat.h"
#include "xv6-user/user.h"

char*
fmtname(char *name)
{
  static char buf[STAT_MAX_NAME+1];
  int len = strlen(name);

  // Return blank-padded name.
  if(len >= STAT_MAX_NAME)
    return name;
  memmove(buf, name, len);
  memset(buf + len, ' ', STAT_MAX_NAME - len);
  buf[STAT_MAX_NAME] = '\0';
  return buf;
}

void
print_permissions(struct stat *st)
{
  printf("%c", st->readable ? 'r' : '-');
  printf("%c", st->writable ? 'w' : '-');
  printf("%c", st->executable ? 'x' : '-');
}

void
ls(char *path, int long_format)
{
  int fd;
  struct stat st;
  char *types[] = {
    [T_DIR]   "DIR ",
    [T_FILE]  "FILE",
  };

  if((fd = open(path, 0)) < 0){
    fprintf(2, "ls: cannot open %s\n", path);
    return;
  }

  if(fstat(fd, &st) < 0){
    fprintf(2, "ls: cannot stat %s\n", path);
    close(fd);
    return;
  }

  if (st.type == T_DIR){
    while(readdir(fd, &st) == 1){
      if (long_format) {
        printf("%c", (st.type == T_DIR) ? 'd' : '-');
        print_permissions(&st);
        printf("\t%s %s\t%d\n", fmtname(st.name), types[st.type], st.size);
      } else {
        printf("%s %s\t%d\n", fmtname(st.name), types[st.type], st.size);
      }
    }
  } else {
    if (long_format) {
      printf("%c", (st.type == T_DIR) ? 'd' : '-');
      print_permissions(&st);
      printf("\t%s %s\t%d\n", fmtname(st.name), types[st.type], st.size);
    } else {
      printf("%s %s\t%d\n", fmtname(st.name), types[st.type], st.size);
    }
  }
  close(fd);
}

int
main(int argc, char *argv[])
{
  int i;
  int long_format = 0;

  // 处理命令行参数
  for (i = 1; i < argc; i++) {
    if (argv[i][0] == '-') {
      if (strcmp(argv[i], "-l") == 0) {
        long_format = 1;
      }
    } else {
      break;
    }
  }

  // 如果没有指定文件路径，默认使用当前目录
  if (i == argc) {
    ls(".", long_format);
    exit(0);
  }

  // 处理指定的文件路径
  for (; i < argc; i++) {
    ls(argv[i], long_format);
  }
  exit(0);
}
